import { StringSelectMenuInteraction, MessageComponentInteraction, ChatInputCommandInteraction, BaseMessageOptions, TextBasedChannel, MessageActionRowComponentData, Message, Client, ButtonStyle, ButtonInteraction } from 'discord.js';
import React, { ReactNode } from 'react';
import { SnakeCasedPropertiesDeep, Except } from 'type-fest';
import { Subject } from 'rxjs';

/**
 * @category Embed
 * @see https://discord.com/developers/docs/resources/channel#embed-object
 */
type EmbedProps = {
    title?: string;
    description?: string;
    url?: string;
    color?: number;
    fields?: Array<{
        name: string;
        value: string;
        inline?: boolean;
    }>;
    author?: {
        name: string;
        url?: string;
        iconUrl?: string;
    };
    thumbnail?: {
        url: string;
    };
    image?: {
        url: string;
    };
    video?: {
        url: string;
    };
    footer?: {
        text: string;
        iconUrl?: string;
    };
    timestamp?: string | number | Date;
    children?: React.ReactNode;
};
/**
 * @category Embed
 * @see https://discord.com/developers/docs/resources/channel#embed-object
 */
declare function Embed(props: EmbedProps): JSX.Element;

type EmbedOptions = SnakeCasedPropertiesDeep<Except<EmbedProps, "timestamp" | "children"> & {
    timestamp?: string;
}>;

/**
 * @category Select
 */
type SelectProps = {
    children?: ReactNode;
    /** Sets the currently selected value */
    value?: string;
    /** Sets the currently selected values, for use with `multiple` */
    values?: string[];
    /** The text shown when no value is selected */
    placeholder?: string;
    /** Set to true to allow multiple selected values */
    multiple?: boolean;
    /**
     * With `multiple`, the minimum number of values that can be selected.
     * When `multiple` is false or not defined, this is always 1.
     *
     * This only limits the number of values that can be received by the user.
     * This does not limit the number of values that can be displayed by you.
     */
    minValues?: number;
    /**
     * With `multiple`, the maximum number of values that can be selected.
     * When `multiple` is false or not defined, this is always 1.
     *
     * This only limits the number of values that can be received by the user.
     * This does not limit the number of values that can be displayed by you.
     */
    maxValues?: number;
    /** When true, the select will be slightly faded, and cannot be interacted with. */
    disabled?: boolean;
    /**
     * Called when the user inputs a selection.
     * Receives the entire select change event,
     * which can be used to create new replies, etc.
     */
    onChange?: (event: StringSelectMenuInteraction) => unknown | Promise<unknown>;
    /**
     * Convenience shorthand for `onChange`, which receives the first selected value.
     */
    onChangeValue?: (value: string, event: StringSelectMenuInteraction) => unknown | Promise<unknown>;
    /**
     * Convenience shorthand for `onChange`, which receives all selected values.
     */
    onChangeMultiple?: (values: string[], event: StringSelectMenuInteraction) => unknown | Promise<unknown>;
};
/**
 * See [the select menu guide](/guides/select-menu) for a usage example.
 * @category Select
 */
declare function Select(props: SelectProps): JSX.Element;

declare class Container<T> {
    private items;
    add(...items: T[]): void;
    addBefore(item: T, before: T): void;
    remove(toRemove: T): void;
    clear(): void;
    find(predicate: (item: T) => boolean): T | undefined;
    findTypeFromTypeguard<U extends T>(typeguard: (item: T) => item is U): U | undefined;
    [Symbol.iterator](): IterableIterator<T>;
}

declare const nodeTypes: readonly ["EmbedAuthor", "EmbedAuthorText", "EmbedField", "EmbedFieldName", "EmbedFieldValue", "EmbedFooter", "EmbedFooterText", "EmbedImage", "EmbedImageText", "EmbedThumbnail", "EmbedThumbnailText", "EmbedTitle", "EmbedTitleText", "EmbedChild", "EmbedOption", "Embed", "ActionRow", "Button", "ButtonLabel", "Link", "LinkLabel", "Select", "SelectLabel", "Option", "OptionDescription", "OptionLabel", "Text", "Node"];
type NodeTypes = typeof nodeTypes[number];
declare class Node<Props> {
    props: Props;
    readonly children: Container<Node<unknown>>;
    type: NodeTypes;
    interaction?: MessageComponentInteraction;
    onComplete?: () => void;
    constructor(props: Props);
    modifyMessageOptions(options: MessageOptions): void;
    handleComponentInteraction(interaction: MessageComponentInteraction, onComplete: () => void): Node<unknown> | undefined;
    completeInteraction(): void;
    get isDeferred(): boolean | undefined;
    get text(): string;
}

type UpdatePayload = {
    action: "stateChange" | "deactivate";
    options: BaseMessageOptions;
} | {
    action: "deferUpdate";
    interaction: MessageComponentInteraction;
    node: Node<unknown>;
    renderer: Renderer;
} | {
    action: "destroy";
} | {
    action: "interactionComplete";
    interaction: MessageComponentInteraction;
    options: BaseMessageOptions;
};
type RendererOptions = {
    type: "interaction";
    interaction: RendererableInteractions;
    ephemeral: boolean;
} | {
    type: "message";
    channel: TextBasedChannel;
};
type RendererableInteractions = MessageComponentInteraction | ChatInputCommandInteraction;
declare class Renderer {
    options: RendererOptions;
    readonly nodes: Container<Node<unknown>>;
    message?: DiscordJSReactMessage;
    active: boolean;
    updates: Subject<UpdatePayload>;
    constructor(options: RendererOptions, initialContent?: React.ReactNode);
    private updateSubscription;
    render(): void;
    deactivate(): void;
    destroy(): void;
    handleComponentInteraction(interaction: MessageComponentInteraction): Node<unknown> | undefined;
    private getMessageOptions;
    private updateMessage;
}

type MessageOptions = {
    content: string;
    embeds: EmbedOptions[];
    actionRows: ActionRow$1[];
};
type ActionRow$1 = ActionRowItem[];
type ActionRowItem = MessageActionRowComponentData;
declare class DiscordJSReactMessage {
    readonly raw: Message;
    readonly renderer: Renderer;
    constructor(raw: Message, renderer: Renderer);
}

type ReacordConfig = {
    /**
     * The max number of active instances.
     * When this limit is exceeded, the oldest instances will be disabled.
     */
    maxInstances?: number;
};
declare class DiscordJSReact {
    readonly client: Client;
    readonly config: ReacordConfig;
    protected renderers: Renderer[];
    constructor(client: Client, config?: ReacordConfig);
    private deactivate;
    private get maxInstances();
    createRenderer(renderer: Renderer, initialContent?: ReactNode): void;
    ephemeralReply(interaction: RendererableInteractions, initialContent: ReactNode): void;
    getMessage(index?: number): DiscordJSReactMessage | undefined;
}

/**
 * Props for an action row
 * @category Action Row
 */
type ActionRowProps = {
    children?: ReactNode;
};
/**
 * An action row is a top-level container for message components.
 *
 * You don't need to use this; Reacord automatically creates action rows for you.
 * But this can be useful if you want a specific layout.
 *
 * ```tsx
 * // put buttons on two separate rows
 * <ActionRow>
 *   <Button label="First" onClick={handleFirst} />
 * </ActionRow>
 * <Button label="Second" onClick={handleSecond} />
 * ```
 *
 * @category Action Row
 * @see https://discord.com/developers/docs/interactions/message-components#action-rows
 */
declare function ActionRow(props: ActionRowProps): JSX.Element;

/**
 * Common props between button-like components
 * @category Button
 */
type ButtonSharedProps = {
    /** The text on the button. Rich formatting (markdown) is not supported here. */
    label?: ReactNode;
    /** When true, the button will be slightly faded, and cannot be clicked. */
    disabled?: boolean;
    /**
     * Renders an emoji to the left of the text.
     * Has to be a literal emoji character (e.g. 🍍),
     * or an emoji code, like `<:plus_one:778531744860602388>`.
     *
     * To get an emoji code, type your emoji in Discord chat
     * with a backslash `\` in front.
     * The bot has to be in the emoji's guild to use it.
     */
    emoji?: string;
};

/**
 * @category Button
 */
type ButtonProps = ButtonSharedProps & {
    /**
     * The style determines the color of the button and signals intent.
     * @see https://discord.com/developers/docs/interactions/message-components#button-object-button-styles
     */
    style?: keyof Omit<typeof ButtonStyle, 'Link'>;
    /**
     * Happens when a user clicks the button.
     */
    onClick: (interaction: ButtonInteraction) => Promise<unknown> | unknown;
};
/**
 * @category Button
 */
declare function Button(props: ButtonProps): JSX.Element;
declare function isButtonNode(node: Node<unknown>): node is ButtonNode;
declare class ButtonNode extends Node<ButtonProps> {
    private customId;
    get text(): string;
    modifyMessageOptions(options: MessageOptions): void;
    handleComponentInteraction(interaction: MessageComponentInteraction, onComplete: () => void): this | undefined;
}

/**
 * @category Embed
 */
type EmbedAuthorProps = {
    name?: ReactNode;
    children?: ReactNode;
    url?: string;
    iconUrl?: string;
};
/**
 * @category Embed
 */
declare function EmbedAuthor(props: EmbedAuthorProps): JSX.Element;

/**
 * @category Embed
 */
type EmbedFieldProps = {
    name: ReactNode;
    value?: ReactNode;
    inline?: boolean;
    children?: ReactNode;
};
/**
 * @category Embed
 */
declare function EmbedField(props: EmbedFieldProps): JSX.Element;

/**
 * @category Embed
 */
type EmbedFooterProps = {
    text?: ReactNode;
    children?: ReactNode;
    iconUrl?: string;
    timestamp?: string | number | Date;
};
/**
 * @category Embed
 */
declare function EmbedFooter({ text, children, ...props }: EmbedFooterProps): JSX.Element;

/**
 * @category Embed
 */
type EmbedImageProps = {
    url: string;
};
/**
 * @category Embed
 */
declare function EmbedImage(props: EmbedImageProps): JSX.Element;

/**
 * @category Embed
 */
type EmbedThumbnailProps = {
    url: string;
};
/**
 * @category Embed
 */
declare function EmbedThumbnail(props: EmbedThumbnailProps): JSX.Element;

/**
 * @category Embed
 */
type EmbedTitleProps = {
    children: ReactNode;
    url?: string;
};
/**
 * @category Embed
 */
declare function EmbedTitle({ children, ...props }: EmbedTitleProps): JSX.Element;

/**
 * @category Link
 */
type LinkProps = ButtonSharedProps & {
    /** The URL the link should lead to */
    url: string;
    /** The link text */
    children?: string;
};
/**
 * @category Link
 */
declare function Link({ label, children, ...props }: LinkProps): JSX.Element;

/**
 * @category Select
 */
type OptionProps = {
    /** The internal value of this option */
    value: string;
    /** The text shown to the user. This takes priority over `children` */
    label?: ReactNode;
    /** The text shown to the user */
    children?: ReactNode;
    /** Description for the option, shown to the user */
    description?: ReactNode;
    /**
     * Renders an emoji to the left of the text.
     *
     * Has to be a literal emoji character (e.g. 🍍),
     * or an emoji code, like `<:plus_one:778531744860602388>`.
     *
     * To get an emoji code, type your emoji in Discord chat
     * with a backslash `\` in front.
     * The bot has to be in the emoji's guild to use it.
     */
    emoji?: string;
};
/**
 * @category Select
 */
declare function Option({ label, children, description, ...props }: OptionProps): JSX.Element;

export { ActionRow, ActionRowProps, Button, ButtonNode, ButtonProps, ButtonSharedProps, DiscordJSReact, Embed, EmbedAuthor, EmbedAuthorProps, EmbedField, EmbedFieldProps, EmbedFooter, EmbedFooterProps, EmbedImage, EmbedImageProps, EmbedProps, EmbedThumbnail, EmbedThumbnailProps, EmbedTitle, EmbedTitleProps, Link, LinkProps, Option, OptionProps, ReacordConfig, Select, SelectProps, isButtonNode };
