// src/discordjs-react.ts
import { Events } from "discord.js";

// src/renderer.ts
import { ComponentType } from "discord.js";
import { concatMap, Subject } from "rxjs";

// src/container.ts
var Container = class {
  items = [];
  add(...items) {
    this.items.push(...items);
  }
  addBefore(item, before) {
    let index = this.items.indexOf(before);
    if (index === -1) {
      index = this.items.length;
    }
    this.items.splice(index, 0, item);
  }
  remove(toRemove) {
    this.items = this.items.filter((item) => item !== toRemove);
  }
  clear() {
    this.items = [];
  }
  find(predicate) {
    return this.items.find(predicate);
  }
  findTypeFromTypeguard(typeguard) {
    for (const item of this.items) {
      if (typeguard(item))
        return item;
    }
  }
  [Symbol.iterator]() {
    return this.items[Symbol.iterator]();
  }
};

// src/helpers/helpers.ts
function omit(subject, keys) {
  const result = {};
  for (const key in subject) {
    if (!keys.includes(key)) {
      result[key] = subject[key];
    }
  }
  return result;
}
function toError(value) {
  return value instanceof Error ? value : new Error(String(value));
}
function raise(error) {
  throw toError(error);
}
function last(array) {
  return array[array.length - 1];
}

// src/reconciler.ts
import ReactReconciler from "react-reconciler";
import { DefaultEventPriority } from "react-reconciler/constants";

// src/node.ts
var embedChildTypes = ["EmbedAuthor", "EmbedAuthorText", "EmbedField", "EmbedFieldName", "EmbedFieldValue", "EmbedFooter", "EmbedFooterText", "EmbedImage", "EmbedImageText", "EmbedThumbnail", "EmbedThumbnailText", "EmbedTitle", "EmbedTitleText"];
var embedTypes = [...embedChildTypes, "EmbedChild", "EmbedOption", "Embed"];
var nodeTypes = [...embedTypes, "ActionRow", "Button", "ButtonLabel", "Link", "LinkLabel", "Select", "SelectLabel", "Option", "OptionDescription", "OptionLabel", "Text", "Node"];
function isNodeTypeguard(node) {
  return nodeTypes.includes(node.type);
}
var Node = class {
  constructor(props) {
    this.props = props;
  }
  children = new Container();
  type = "Node";
  interaction = void 0;
  onComplete = void 0;
  modifyMessageOptions(options) {
  }
  handleComponentInteraction(interaction, onComplete) {
    return void 0;
  }
  completeInteraction() {
    this.interaction = void 0;
    if (this.onComplete)
      this.onComplete();
  }
  get isDeferred() {
    var _a;
    return (_a = this.interaction) == null ? void 0 : _a.deferred;
  }
  get text() {
    return [...this.children].map((child) => child.text).join("");
  }
};

// src/components/text-node.ts
function isTextNodeTypeguard(node) {
  return node.type === "Text";
}
var TextNode = class extends Node {
  type = "Text";
  modifyMessageOptions(options) {
    options.content = options.content + this.props;
  }
  get text() {
    return this.props;
  }
};

// src/reconciler.ts
var config = {
  supportsMutation: true,
  supportsPersistence: false,
  supportsHydration: false,
  isPrimaryRenderer: true,
  scheduleTimeout: global.setTimeout,
  cancelTimeout: global.clearTimeout,
  noTimeout: -1,
  // eslint-disable-next-line unicorn/no-null
  getRootHostContext: () => null,
  getChildHostContext: (parentContext) => parentContext,
  createInstance: (type, props) => {
    if (type !== "reacord-element") {
      raise(`Unknown element type: ${type}`);
    }
    if (typeof props.createNode !== "function") {
      raise(`Missing createNode function`);
    }
    const node = props.createNode(props.props);
    if (!isNodeTypeguard(node)) {
      raise(`createNode function did not return a Node`);
    }
    return node;
  },
  createTextInstance: (text) => new TextNode(text),
  shouldSetTextContent: () => false,
  detachDeletedInstance: (instance) => {
  },
  beforeActiveInstanceBlur: () => {
  },
  afterActiveInstanceBlur: () => {
  },
  // eslint-disable-next-line unicorn/no-null
  getInstanceFromNode: (node) => null,
  // eslint-disable-next-line unicorn/no-null
  getInstanceFromScope: (scopeInstance) => null,
  clearContainer: (renderer) => {
    renderer.nodes.clear();
  },
  appendChildToContainer: (renderer, child) => {
    renderer.nodes.add(child);
  },
  removeChildFromContainer: (renderer, child) => {
    renderer.nodes.remove(child);
  },
  insertInContainerBefore: (renderer, child, before) => {
    renderer.nodes.addBefore(child, before);
  },
  appendInitialChild: (parent, child) => {
    parent.children.add(child);
  },
  appendChild: (parent, child) => {
    parent.children.add(child);
  },
  removeChild: (parent, child) => {
    parent.children.remove(child);
  },
  insertBefore: (parent, child, before) => {
    parent.children.addBefore(child, before);
  },
  prepareUpdate: () => true,
  commitUpdate: (node, payload, type, oldProps, newProps) => {
    node.props = newProps.props;
  },
  commitTextUpdate: (node, oldText, newText) => {
    node.props = newText;
  },
  // eslint-disable-next-line unicorn/no-null
  prepareForCommit: () => null,
  resetAfterCommit: (renderer) => {
    renderer.render();
  },
  prepareScopeUpdate: (scopeInstance, instance) => {
  },
  preparePortalMount: () => raise("Portals are not supported"),
  getPublicInstance: () => raise("Refs are currently not supported"),
  finalizeInitialChildren: () => false,
  getCurrentEventPriority: () => DefaultEventPriority
};
var reconciler = ReactReconciler(config);

// src/message.ts
var DiscordJSReactMessage = class {
  constructor(raw, renderer) {
    this.raw = raw;
    this.renderer = renderer;
  }
};

// src/renderer.ts
function getNextActionRow(options) {
  let actionRow = last(options.actionRows);
  const firstItem = actionRow == null ? void 0 : actionRow[0];
  if (actionRow == void 0 || actionRow.length >= 5 || firstItem && "type" in firstItem && firstItem.type === ComponentType.StringSelect) {
    actionRow = [];
    options.actionRows.push(actionRow);
  }
  return actionRow;
}
var LOADING_EMOJI = "<a:loading:1081524604419453028>";
var Renderer = class {
  constructor(options, initialContent) {
    this.options = options;
    const container = reconciler.createContainer(
      this,
      0,
      // eslint-disable-next-line unicorn/no-null
      null,
      false,
      // eslint-disable-next-line unicorn/no-null
      null,
      "reacord",
      () => {
      },
      // eslint-disable-next-line unicorn/no-null
      null
    );
    if (initialContent) {
      reconciler.updateContainer(initialContent, container);
    }
  }
  nodes = new Container();
  message;
  active = true;
  updates = new Subject();
  updateSubscription = this.updates.pipe(concatMap((payload) => this.updateMessage(payload))).subscribe({ error: console.error });
  render() {
    if (!this.active) {
      console.warn("Attempted to update a deactivated message");
      return;
    }
    this.updates.next({
      options: this.getMessageOptions(),
      action: "stateChange"
    });
  }
  deactivate() {
    this.active = false;
    this.updates.next({
      options: this.getMessageOptions(),
      action: "deactivate"
    });
  }
  destroy() {
    this.active = false;
    this.updates.next({ action: "destroy" });
  }
  handleComponentInteraction(interaction) {
    for (const node of this.nodes) {
      const handler = node.handleComponentInteraction(interaction, () => {
        this.updates.next({
          options: this.getMessageOptions(),
          action: "interactionComplete",
          interaction
        });
      });
      if (handler) {
        setTimeout(() => {
          this.updates.next({ action: "deferUpdate", interaction, node: handler, renderer: this });
        }, 2e3);
        return handler;
      }
    }
    return void 0;
  }
  getMessageOptions() {
    const options = {
      content: "",
      embeds: [],
      actionRows: []
    };
    for (const node of this.nodes) {
      node.modifyMessageOptions(options);
    }
    return {
      components: options.actionRows.map((row) => ({
        type: ComponentType.ActionRow,
        components: row
      })),
      content: options.content,
      embeds: options.embeds
    };
  }
  async updateMessage(payload) {
    var _a, _b;
    if (payload.action === "destroy") {
      this.updateSubscription.unsubscribe();
      await ((_a = this.message) == null ? void 0 : _a.raw.delete());
      return;
    }
    if (payload.action === "deactivate") {
      this.updateSubscription.unsubscribe();
      await ((_b = this.message) == null ? void 0 : _b.raw.edit({
        ...payload.options,
        // TODO: Disable all components
        components: []
      }));
      return;
    }
    if (payload.action === "deferUpdate") {
      if (payload.interaction.deferred || payload.interaction.replied)
        return;
      await payload.interaction.deferUpdate();
      payload.renderer.render();
      return;
    }
    if (payload.action === "stateChange" && this.message) {
      if (this.options.type === "interaction") {
        await this.options.interaction.editReply(payload.options);
      } else {
        await this.message.raw.edit(payload.options);
      }
      return;
    }
    if (payload.action === "interactionComplete") {
      if (payload.interaction.deferred) {
        await payload.interaction.editReply(payload.options);
        return;
      } else {
        await payload.interaction.update(payload.options);
      }
      return;
    }
    if (this.options.type === "interaction") {
      const intr = this.options.interaction;
      const created = await intr.reply({
        ephemeral: this.options.ephemeral,
        fetchReply: true,
        ...payload.options
      });
      this.message = new DiscordJSReactMessage(created, this);
    }
  }
};

// src/discordjs-react.ts
var DiscordJSReact = class {
  constructor(client, config2 = {}) {
    this.client = client;
    this.config = config2;
    client.on(Events.InteractionCreate, (interaction) => {
      if (!interaction.isMessageComponent())
        return;
      for (const renderer of this.renderers) {
        renderer.handleComponentInteraction(interaction);
      }
    });
  }
  // Maps the interaction id to the renderer
  renderers = [];
  deactivate(renderer) {
    this.renderers = this.renderers.filter((it) => it !== renderer);
    renderer.deactivate();
  }
  get maxInstances() {
    return this.config.maxInstances ?? 50;
  }
  createRenderer(renderer, initialContent) {
    if (this.renderers.length >= this.maxInstances) {
      this.deactivate(this.renderers[0]);
    }
    this.renderers.push(renderer);
    if (initialContent) {
      renderer.render();
    }
  }
  ephemeralReply(interaction, initialContent) {
    this.createRenderer(new Renderer({
      type: "interaction",
      interaction,
      ephemeral: true
    }, initialContent));
  }
  getMessage(index = 0) {
    var _a;
    return (_a = this.renderers.at(index)) == null ? void 0 : _a.message;
  }
};

// src/components/action-row.tsx
import React2 from "react";

// src/element.ts
import React from "react";
function DiscordJSReactElement(props) {
  return React.createElement("reacord-element", props);
}

// src/components/action-row.tsx
function ActionRow2(props) {
  return /* @__PURE__ */ React2.createElement(DiscordJSReactElement, { props, createNode: () => new ActionRowNode(props) }, props.children);
}
var ActionRowNode = class extends Node {
  modifyMessageOptions(options) {
    options.actionRows.push([]);
    for (const child of this.children) {
      child.modifyMessageOptions(options);
    }
  }
  handleComponentInteraction(interaction, onComplete) {
    for (const child of this.children) {
      const handler = child.handleComponentInteraction(interaction, onComplete);
      if (handler) {
        return handler;
      }
    }
    return void 0;
  }
};

// src/components/button.tsx
import { ButtonStyle, ComponentType as ComponentType2 } from "discord.js";
import { randomUUID } from "node:crypto";
import React3 from "react";
function Button(props) {
  return /* @__PURE__ */ React3.createElement(DiscordJSReactElement, { props, createNode: () => new ButtonNode(props) }, /* @__PURE__ */ React3.createElement(DiscordJSReactElement, { props: {}, createNode: () => new ButtonLabelNode({}) }, props.label));
}
function isButtonNode(node) {
  return node.type === "Button";
}
var ButtonNode = class extends Node {
  customId = randomUUID();
  // this has text children, but buttons themselves shouldn't yield text
  // eslint-disable-next-line class-methods-use-this
  get text() {
    return "";
  }
  modifyMessageOptions(options) {
    var _a;
    const buttonStyle = this.props.style;
    const opts = {
      type: ComponentType2.Button,
      customId: this.customId,
      style: buttonStyle ? ButtonStyle[buttonStyle] : ButtonStyle.Secondary,
      disabled: this.props.disabled || this.isDeferred !== void 0,
      emoji: this.isDeferred ? LOADING_EMOJI : this.props.emoji,
      label: (_a = this.children.findTypeFromTypeguard(isButtonLabelNode)) == null ? void 0 : _a.text
    };
    getNextActionRow(options).push(opts);
  }
  handleComponentInteraction(interaction, onComplete) {
    const shouldHandleInteraction = interaction.isButton() && interaction.customId === this.customId;
    if (!shouldHandleInteraction)
      return void 0;
    this.interaction = interaction;
    this.onComplete = onComplete;
    Promise.resolve(this.props.onClick(interaction)).then(
      () => this.completeInteraction()
    );
    return this;
  }
};
function isButtonLabelNode(node) {
  return node.type === "ButtonLabel";
}
var ButtonLabelNode = class extends Node {
  type = "ButtonLabel";
};

// src/components/embeds/embed.tsx
import React4 from "react";

// src/components/embeds/embed-child.ts
var EmbedChildNode = class extends Node {
};

// src/helpers/convert-case.ts
import { camelCase, isObject, snakeCase } from "lodash-es";
function convertKeyCaseDeep(input, convertKey) {
  if (!isObject(input)) {
    return input;
  }
  if (Array.isArray(input)) {
    return input.map(
      (item) => convertKeyCaseDeep(item, convertKey)
    );
  }
  const output = {};
  for (const [key, value] of Object.entries(input)) {
    output[convertKey(key)] = convertKeyCaseDeep(value, convertKey);
  }
  return output;
}
function snakeCaseDeep(input) {
  return convertKeyCaseDeep(input, snakeCase);
}

// src/components/embeds/embed.tsx
function Embed(props) {
  return /* @__PURE__ */ React4.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedNode(props) }, props.children);
}
var EmbedNode = class extends Node {
  modifyMessageOptions(options) {
    const embed = {
      ...snakeCaseDeep(omit(this.props, ["children", "timestamp"])),
      timestamp: this.props.timestamp ? new Date(this.props.timestamp).toISOString() : void 0
    };
    for (const child of this.children) {
      if (child instanceof EmbedChildNode) {
        child.modifyEmbedOptions(embed);
      }
      if (isTextNodeTypeguard(child)) {
        embed.description = (embed.description || "") + child.props;
      }
    }
    options.embeds.push(embed);
  }
};

// src/components/embeds/embed-author.tsx
import React5 from "react";
function EmbedAuthor(props) {
  return /* @__PURE__ */ React5.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedAuthorNode(props) }, /* @__PURE__ */ React5.createElement(DiscordJSReactElement, { props: {}, createNode: () => new AuthorTextNode({}) }, props.name ?? props.children));
}
var EmbedAuthorNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a;
    options.author = {
      name: ((_a = this.children.findTypeFromTypeguard(isAuthorTextNode)) == null ? void 0 : _a.text) ?? "",
      url: this.props.url,
      icon_url: this.props.iconUrl
    };
  }
};
function isAuthorTextNode(node) {
  return node.type === "EmbedAuthorText";
}
var AuthorTextNode = class extends Node {
  type = "EmbedAuthorText";
};

// src/components/embeds/embed-field.tsx
import React6 from "react";
function EmbedField(props) {
  return /* @__PURE__ */ React6.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedFieldNode(props) }, /* @__PURE__ */ React6.createElement(DiscordJSReactElement, { props: {}, createNode: () => new FieldNameNode({}) }, props.name), /* @__PURE__ */ React6.createElement(DiscordJSReactElement, { props: {}, createNode: () => new FieldValueNode({}) }, props.value || props.children));
}
var EmbedFieldNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a, _b;
    options.fields ??= [];
    options.fields.push({
      name: ((_a = this.children.findTypeFromTypeguard(isFieldNameNode)) == null ? void 0 : _a.text) ?? "",
      value: ((_b = this.children.findTypeFromTypeguard(isFieldValueNode)) == null ? void 0 : _b.text) ?? "",
      inline: this.props.inline
    });
  }
};
function isFieldNameNode(node) {
  return node.type === "EmbedFieldName";
}
var FieldNameNode = class extends Node {
  type = "EmbedFieldName";
};
function isFieldValueNode(node) {
  return node.type === "EmbedFieldValue";
}
var FieldValueNode = class extends Node {
  type = "EmbedFieldValue";
};

// src/components/embeds/embed-footer.tsx
import React7 from "react";
function EmbedFooter({ text, children, ...props }) {
  return /* @__PURE__ */ React7.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedFooterNode(props) }, /* @__PURE__ */ React7.createElement(DiscordJSReactElement, { props: {}, createNode: () => new FooterTextNode({}) }, text ?? children));
}
var EmbedFooterNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a;
    options.footer = {
      text: ((_a = this.children.findTypeFromTypeguard(isFooterTextNode)) == null ? void 0 : _a.text) ?? "",
      icon_url: this.props.iconUrl
    };
    options.timestamp = this.props.timestamp ? new Date(this.props.timestamp).toISOString() : void 0;
  }
};
function isFooterTextNode(node) {
  return node.type === "EmbedFooterText";
}
var FooterTextNode = class extends Node {
  type = "EmbedFooterText";
};

// src/components/embeds/embed-image.tsx
import React8 from "react";
function EmbedImage(props) {
  return /* @__PURE__ */ React8.createElement(
    DiscordJSReactElement,
    {
      props,
      createNode: () => new EmbedImageNode(props)
    }
  );
}
var EmbedImageNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    options.image = { url: this.props.url };
  }
};

// src/components/embeds/embed-thumbnail.tsx
import React9 from "react";
function EmbedThumbnail(props) {
  return /* @__PURE__ */ React9.createElement(
    DiscordJSReactElement,
    {
      props,
      createNode: () => new EmbedThumbnailNode(props)
    }
  );
}
var EmbedThumbnailNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    options.thumbnail = { url: this.props.url };
  }
};

// src/components/embeds/embed-title.tsx
import React10 from "react";
function EmbedTitle({ children, ...props }) {
  return /* @__PURE__ */ React10.createElement(DiscordJSReactElement, { props, createNode: () => new EmbedTitleNode(props) }, /* @__PURE__ */ React10.createElement(DiscordJSReactElement, { props: {}, createNode: () => new TitleTextNode({}) }, children));
}
var EmbedTitleNode = class extends EmbedChildNode {
  modifyEmbedOptions(options) {
    var _a;
    options.title = ((_a = this.children.findTypeFromTypeguard(
      isTitleTextNode
    )) == null ? void 0 : _a.text) ?? "";
    options.url = this.props.url;
  }
};
function isTitleTextNode(node) {
  return node.type === "EmbedTitleText";
}
var TitleTextNode = class extends Node {
  type = "EmbedTitleText";
};

// src/components/link.tsx
import { ButtonStyle as ButtonStyle2, ComponentType as ComponentType3 } from "discord.js";
import React11 from "react";
function Link({ label, children, ...props }) {
  return /* @__PURE__ */ React11.createElement(DiscordJSReactElement, { props, createNode: () => new LinkNode(props) }, /* @__PURE__ */ React11.createElement(DiscordJSReactElement, { props: {}, createNode: () => new LinkTextNode({}) }, label || children));
}
var LinkNode = class extends Node {
  modifyMessageOptions(options) {
    var _a;
    getNextActionRow(options).push({
      type: ComponentType3.Button,
      style: ButtonStyle2.Link,
      disabled: this.props.disabled,
      emoji: this.props.emoji,
      label: ((_a = this.children.findTypeFromTypeguard(isLinkTextNode)) == null ? void 0 : _a.text) ?? "",
      url: this.props.url
    });
  }
};
function isLinkTextNode(node) {
  return node.type === "LinkLabel";
}
var LinkTextNode = class extends Node {
  type = "LinkLabel";
};

// src/components/option.tsx
import React12 from "react";

// src/components/option-node.ts
function isOptionNodeTypeguard(node) {
  return node.type === "Option";
}
var OptionNode = class extends Node {
  type = "Option";
  get options() {
    var _a, _b;
    return {
      label: ((_a = this.children.findTypeFromTypeguard(isOptionLabelNodeTypeguard)) == null ? void 0 : _a.text) ?? "",
      value: this.props.value,
      description: ((_b = this.children.findTypeFromTypeguard(isOptionValueNodeTypeguard)) == null ? void 0 : _b.text) ?? "",
      emoji: this.props.emoji
    };
  }
};
function isOptionLabelNodeTypeguard(node) {
  return node.type === "OptionLabel";
}
var OptionLabelNode = class extends Node {
  type = "OptionLabel";
};
function isOptionValueNodeTypeguard(node) {
  return node.type === "OptionDescription";
}
var OptionDescriptionNode = class extends Node {
  type = "OptionDescription";
};

// src/components/option.tsx
function Option({
  label,
  children,
  description,
  ...props
}) {
  return /* @__PURE__ */ React12.createElement(DiscordJSReactElement, { props, createNode: () => new OptionNode(props) }, (label !== void 0 || children !== void 0) && /* @__PURE__ */ React12.createElement(DiscordJSReactElement, { props: {}, createNode: () => new OptionLabelNode({}) }, label || children), description !== void 0 && /* @__PURE__ */ React12.createElement(
    DiscordJSReactElement,
    {
      props: {},
      createNode: () => new OptionDescriptionNode({})
    },
    description
  ));
}

// src/components/select.tsx
import { ComponentType as ComponentType4 } from "discord.js";
import { randomUUID as randomUUID2 } from "node:crypto";
import React13 from "react";
function Select(props) {
  return /* @__PURE__ */ React13.createElement(DiscordJSReactElement, { props, createNode: () => new SelectNode(props) }, props.children);
}
var SelectNode = class extends Node {
  customId = randomUUID2();
  modifyMessageOptions(message) {
    const actionRow = [];
    message.actionRows.push(actionRow);
    const {
      multiple,
      value,
      values,
      minValues = 0,
      maxValues = 25,
      children,
      onChange,
      onChangeValue,
      onChangeMultiple,
      ...props
    } = this.props;
    const options = [...this.children].filter(
      isOptionNodeTypeguard
    ).map((node) => ({
      ...node.options,
      default: value === node.options.value && !this.isDeferred
    }));
    const item = {
      ...props,
      type: ComponentType4.StringSelect,
      customId: this.customId,
      options,
      disabled: this.isDeferred,
      placeholder: this.isDeferred ? "Applying selection..." : props.placeholder
    };
    if (multiple) {
      item.minValues = minValues;
      item.maxValues = maxValues;
    }
    if (!multiple && value != void 0) {
    }
    actionRow.push(item);
  }
  handleComponentInteraction(interaction, onComplete) {
    var _a, _b, _c, _d, _e, _f;
    const isSelectInteraction = interaction.isStringSelectMenu() && interaction.customId === this.customId && !this.props.disabled;
    if (!isSelectInteraction)
      return void 0;
    this.interaction = interaction;
    this.interaction = interaction;
    this.onComplete = onComplete;
    const firstValue = interaction.values.at(0);
    Promise.all([
      (_b = (_a = this.props).onChangeMultiple) == null ? void 0 : _b.call(_a, interaction.values, interaction),
      firstValue ? (_d = (_c = this.props).onChangeValue) == null ? void 0 : _d.call(_c, firstValue, interaction) : void 0,
      (_f = (_e = this.props).onChange) == null ? void 0 : _f.call(_e, interaction)
    ]).then(
      () => this.completeInteraction()
    );
    return this;
  }
};
export {
  ActionRow2 as ActionRow,
  Button,
  ButtonNode,
  DiscordJSReact,
  Embed,
  EmbedAuthor,
  EmbedField,
  EmbedFooter,
  EmbedImage,
  EmbedThumbnail,
  EmbedTitle,
  Link,
  Option,
  Select,
  isButtonNode
};
//# sourceMappingURL=index.mjs.map